@@ -60,6 +60,14 @@ process.inputs.items <- function(model, obj, X=NULL, X_col=NULL, X_val=NULL, wei
         !NROW(processed_I$U) && !NROW(processed_I$U_val) &&
         !NROW(processed_I_bin$U))
         stop("Inputs contain no data.")
+
+    if (!NROW(processed_X$X) && NROW(obj$precomputed$A) && obj$info$NA_as_zero) {
+        processed_X$n <- ncol(obj$matrices$A)
+    }
+
+    if (!NROW(processed_I$U) && NROW(obj$matrices$D) && obj$info$NA_as_zero_item) {
+        processed_I$p <- nrow(obj$matrices$D)
+    }
     
     return(list(
         processed_X = processed_X,
@@ -178,6 +186,7 @@ item_factors <- function(model, X=NULL, X_col=NULL, X_val=NULL,
                           model$info$k, model$info$k_item, model$info$k_user, model$info$k_main,
                           lambda, l1_lambda,
                           model$info$scale_lam, model$info$scale_lam_sideinfo,
+                          model$info$scale_bias_const, model$matrices$scaling_biasB,
                           model$info$w_main, model$info$w_item, model$info$w_implicit,
                           NCOL(model$matrices$A), TRUE,
                           numeric(),
@@ -187,6 +196,7 @@ item_factors <- function(model, X=NULL, X_col=NULL, X_val=NULL,
                           numeric(),
                           numeric(),
                           numeric(),
+                          numeric(),
                           numeric())
     } else if ("CMF_implicit" %in% class(model)) {
         b_vec <- numeric(model$info$k_item + model$info$k + model$info$k_main)
@@ -207,6 +217,7 @@ item_factors <- function(model, X=NULL, X_col=NULL, X_val=NULL,
                           model$info$apply_log_transf,
                           numeric(),
                           numeric(),
+                          numeric(),
                           numeric())
     } else if ("ContentBased" %in% class(model)) {
         b_vec <- numeric(model$info$k)
@@ -445,6 +456,7 @@ predict_new_items <- function(model, user, item=NULL,
                           model$info$k, model$info$k_item, model$info$k_user, model$info$k_main,
                           swap.lambda(model$info$lambda), swap.lambda(model$info$l1_lambda),
                           model$info$scale_lam, model$info$scale_lam_sideinfo,
+                          model$info$scale_bias_const, model$matrices$scaling_biasB,
                           model$info$w_main, model$info$w_item, model$info$w_implicit,
                           NCOL(model$matrices$A), model$info$include_all_X,
                           numeric(),
@@ -455,6 +467,7 @@ predict_new_items <- function(model, user, item=NULL,
                           numeric(),
                           numeric(),
                           numeric(),
+                          numeric(),
                           model$info$nthreads)
         check.ret.code(ret_code)
         ret_code <- .Call("call_predict_X_old_collective_explicit",
@@ -488,6 +501,7 @@ predict_new_items <- function(model, user, item=NULL,
                           numeric(),
                           numeric(),
                           numeric(),
+                          numeric(),
                           model$info$nthreads)
         check.ret.code(ret_code)
         ret_code <- .Call("call_predict_X_old_collective_implicit",
