@@ -87,14 +87,15 @@ SEXP prepare_RhpcBLASctl_Call()
     int threw_err = 0;
     const char *arg_names[] = {"package", "quietly", ""};
     int n_protected = 0;
-    SEXP R_str_RhpcBLASctl = PROTECT(Rf_ScalarString(Rf_mkChar("RhpcBLASctl")));
+    SEXP R_ch_RhpcBLASctl = PROTECT(Rf_mkChar("RhpcBLASctl"));
+    SEXP R_str_RhpcBLASctl = PROTECT(Rf_ScalarString(R_ch_RhpcBLASctl));
     SEXP R_expr_requireNamespace = PROTECT(Rf_install("requireNamespace"));
     SEXP R_exptr_do_call = PROTECT(Rf_install("do.call"));
     SEXP call_args = PROTECT(Rf_mkNamed(VECSXP, arg_names));
     SET_VECTOR_ELT(call_args, 0, R_str_RhpcBLASctl);
     SET_VECTOR_ELT(call_args, 1, Rf_ScalarLogical(1));
     SEXP R_expr_call = PROTECT(Rf_lang3(R_exptr_do_call, R_expr_requireNamespace, call_args));
-    n_protected = 5;
+    n_protected = 6;
     SEXP R_has_RhpcBLASctl = R_tryEvalSilent(R_expr_call, R_GlobalEnv, &threw_err);
     if (threw_err) {
         UNPROTECT(n_protected);
@@ -686,6 +687,9 @@ SEXP call_fit_offsets_explicit_lbfgs
     SEXP TransBtBinvBt
 )
 {
+    #if defined(WRAPPED_GELSD) && !defined(USE_FLOAT)
+    GELSD_free_inputs = false;
+    #endif
     SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
 
     double lambda_ = REAL(lam)[0];
@@ -766,6 +770,9 @@ SEXP call_fit_offsets_explicit_als
     SEXP TransBtBinvBt
 )
 {
+    #if defined(WRAPPED_GELSD) && !defined(USE_FLOAT)
+    GELSD_free_inputs = false;
+    #endif
     SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
 
     double *weight = NULL;
@@ -831,6 +838,9 @@ SEXP call_fit_offsets_implicit_als
     SEXP BtB
 )
 {
+    #if defined(WRAPPED_GELSD) && !defined(USE_FLOAT)
+    GELSD_free_inputs = false;
+    #endif
     SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
 
     int retval = fit_offsets_implicit_als(
@@ -1557,6 +1567,8 @@ SEXP call_predict_X_old_collective_explicit
     SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+
     int retval = predict_X_old_collective_explicit(
         INTEGER(row), INTEGER(col), REAL(predicted), (size_t) Rf_xlength(predicted),
         REAL(A), get_ptr(biasA),
@@ -1567,6 +1579,8 @@ SEXP call_predict_X_old_collective_explicit
         Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -1580,6 +1594,8 @@ SEXP call_predict_X_old_collective_implicit
     SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+
     int retval = predict_X_old_collective_implicit(
         INTEGER(row), INTEGER(col), REAL(predicted), (size_t) Rf_xlength(predicted),
         REAL(A),
@@ -1589,6 +1605,8 @@ SEXP call_predict_X_old_collective_implicit
         Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -1600,6 +1618,8 @@ SEXP call_predict_X_old_most_popular
     SEXP m, SEXP n
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+
     int retval = predict_X_old_most_popular(
         INTEGER(row), INTEGER(col), REAL(predicted), (size_t) Rf_xlength(predicted),
         get_ptr(biasA), REAL(biasB),
@@ -1607,6 +1627,8 @@ SEXP call_predict_X_old_most_popular
         Rf_asInteger(m), Rf_asInteger(n)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -1626,6 +1648,8 @@ SEXP call_predict_X_old_content_based
     SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+
     int retval = predict_X_old_content_based(
         REAL(predicted), (size_t) Rf_xlength(predicted),
         Rf_asInteger(m_new), Rf_asInteger(k),
@@ -1641,6 +1665,8 @@ SEXP call_predict_X_old_content_based
         Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -1655,6 +1681,8 @@ SEXP call_predict_X_old_offsets_explicit
     SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+
     int retval = predict_X_old_offsets_explicit(
         INTEGER(row), INTEGER(col), REAL(predicted), (size_t) Rf_xlength(predicted),
         REAL(Am), get_ptr(biasA),
@@ -1665,6 +1693,8 @@ SEXP call_predict_X_old_offsets_explicit
         Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -1678,6 +1708,8 @@ SEXP call_predict_X_old_offsets_implicit
     SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+
     int retval = predict_X_old_offsets_implicit(
         INTEGER(row), INTEGER(col), REAL(predicted), (size_t) Rf_xlength(predicted),
         REAL(Am),
@@ -1687,6 +1719,8 @@ SEXP call_predict_X_old_offsets_implicit
         Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -2025,6 +2059,8 @@ SEXP call_topN_old_collective_explicit
     SEXP n, SEXP n_max, SEXP include_all_X, SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+
     int retval = topN_old_collective_explicit(
         REAL(a_vec), Rf_asReal(a_bias),
         (double*)NULL, (double*)NULL, 0,
@@ -2038,6 +2074,8 @@ SEXP call_topN_old_collective_explicit
         (int) Rf_xlength(outp_ix), Rf_asInteger(n), Rf_asInteger(n_max), (bool) Rf_asLogical(include_all_X), Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -2052,6 +2090,8 @@ SEXP call_topN_old_collective_implicit
     SEXP n, SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+    
     int retval = topN_old_collective_implicit(
         REAL(a_vec),
         (double*)NULL, 0,
@@ -2063,6 +2103,8 @@ SEXP call_topN_old_collective_implicit
         (int) Rf_xlength(outp_ix), Rf_asInteger(n), Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -2078,6 +2120,8 @@ SEXP call_topN_old_most_popular
     SEXP n
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+    
     int retval = topN_old_most_popular(
         (bool) Rf_asLogical(user_bias),
         Rf_asReal(a_bias),
@@ -2090,6 +2134,8 @@ SEXP call_topN_old_most_popular
         (int) Rf_xlength(outp_ix), Rf_asInteger(n)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -2106,6 +2152,8 @@ SEXP call_topN_old_content_based
     SEXP n, SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+    
     int retval = topN_old_content_based(
         REAL(a_vec), Rf_asReal(a_bias),
         (double*)NULL, (double*)NULL, 0,
@@ -2119,6 +2167,8 @@ SEXP call_topN_old_content_based
         (int) Rf_xlength(outp_ix), Rf_asInteger(n), Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -2135,6 +2185,8 @@ SEXP call_topN_old_offsets_explicit
     SEXP n, SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+    
     int retval = topN_old_offsets_explicit(
         REAL(a_vec), Rf_asReal(a_bias),
         (double*)NULL, (double*)NULL, 0,
@@ -2148,6 +2200,8 @@ SEXP call_topN_old_offsets_explicit
         (int) Rf_xlength(outp_ix), Rf_asInteger(n), Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
@@ -2162,6 +2216,8 @@ SEXP call_topN_old_offsets_implicit
     SEXP n, SEXP nthreads
 )
 {
+    SEXP global_lst = PROTECT(prepare_RhpcBLASctl_Call()); ptr_glob_lst = &global_lst;
+    
     int retval = topN_old_offsets_implicit(
         REAL(a_vec),
         (double*)NULL, 0,
@@ -2173,6 +2229,8 @@ SEXP call_topN_old_offsets_implicit
         (int) Rf_xlength(outp_ix), Rf_asInteger(n), Rf_asInteger(nthreads)
     );
 
+    cleanup_RhpcBLASctl_Call();
+
     return Rf_ScalarInteger(retval);
 }
 
