@@ -1237,7 +1237,7 @@ void custom_syr(const int_t n, const real_t alpha, const real_t *restrict x, rea
     real_t *restrict Arow;
     for (int i = 0; i < n; i++) {
         temp = alpha*x[i];
-        Arow = A + (size_t)i*lda;
+        Arow = A + (size_t)i*(size_t)lda;
         for (int j = i; j < n; j++)
             Arow[j] += temp*x[j];
     }
@@ -1293,3 +1293,28 @@ void set_blas_threads(int nthreads_set, int *nthreads_curr)
     omp_set_num_threads(nthreads_set);
     #endif
 }
+
+
+#if defined(_FOR_R) && defined(WRAPPED_GELSD) && !defined(USE_FLOAT)
+SEXP wrapper_GELSD(void *data)
+{
+    Args_to_GELSD *data_ = (Args_to_GELSD*)data;
+    tgelsd_(data_->m, data_->n, data_->nrhs,
+            data_->A, data_->lda, data_->B, data_->ldb,
+            data_->S, data_->rcond, data_->rank,
+            data_->work, data_->lwork, data_->iwork,
+            data_->info);
+    return R_NilValue;
+}
+
+void clean_after_GELSD(void *cdata, Rboolean jump)
+{
+    if (jump)
+    {
+        PointersToFree *cdata_ = (PointersToFree*)cdata;
+        for (size_t ix = 0; ix < cdata_->n_pointers; ix++)
+            free(cdata_->pointers[ix]);
+        GELSD_free_inputs = false;
+    }
+}
+#endif
